/// NScripter向けプラグインのメイン関数の定義です。
/**
	@file szpd.d
	@author senzogawa
**/
module szpddll;

import std.c.windows.windows;
import std.file;
import std.path;
import std.string;
import szpd.plugin;
import szpd.result;
import szpd.command;

const char[] DLL_VERSION = "0.20";

extern(C)
{
	/// NScripterから呼び出される関数。
	export BOOL NScrPlugInMain(HINSTANCE hinstance, HWND hwnd, char *param, int *ret_int, char *ret_str)
	{
		Plugin plugin = new Plugin(param, ret_int, ret_str);
		// パラメータの有無をチェックする。
		if(plugin.isNoParam())
		{
			plugin.setReturnValue(Result.ERROR, "エラー:パラメータがありません。");
			return false;
		}
		
		// パラメータに基づき、処理を実行する。
		switch(plugin.command())
		{
		// DLLのバージョン情報
		case "version":
			plugin.setReturnValue(Result.SUCCESS, DLL_VERSION);
			break;

		// フォルダ作成
		case "mkdir":
			try
			{
				Result result = makeDirectory(plugin.argument());
				plugin.setReturnValue(result.num, result.str);
			}
			catch(Exception exception)
			{
				plugin.setException(exception);
			}
			break;
		
		// 設定の書き出し
		case "setini":
			try
			{
				char[][] args = split(plugin.argument(), ",");
				Result result;
				if(args.length < 3)
				{
					result.set(Result.ERROR, "エラー:引数が足りません。");
				}
				else if(args.length == 3)
				{
					result = setIni(args[0], args[1], args[2], "");
				}
				else
				{
					result = setIni(args[0], args[1], args[2], args[3]);
				}
				plugin.setReturnValue(result.num, result.str);
			}
			catch(Exception exception)
			{
				plugin.setException(exception);
			}
			break;
		
		// 設定の読み込み
		case "getini":
			try
			{
				char[][] args = split(plugin.argument(), ",");
				Result result;
				if(args.length < 3)
				{
					result.set(Result.ERROR, "エラー:引数が足りません。");
				}
				else
				{
					result = getIni(args[0], args[1], args[2]);
				}
				plugin.setReturnValue(result.num, result.str);
			}
			catch(Exception exception)
			{
				plugin.setException(exception);
			}
			break;
		debug
		{
		// ログ吐き出し
		case "putlog":
			try
			{
				Result result = putlog(plugin.argument());
				plugin.setReturnValue(result.num, result.str);
			}
			catch(Exception exception)
			{
				plugin.setException(exception);
			}
			break;
		} // debug end
		
		// その他はとりあえずエラー
		default:
			plugin.setReturnValue(Result.ERROR, "エラー:不正なコマンドです。");
			break;
		}
		
		return false;
	}
}
