/// 文字列変換クラスの定義です。
/**
	@file charcnv.d
	@author senzogawa
**/

module szpd.charcnv;

private import std.utf;

/// 文字コード変換クラス。
/**
	変換処理はやねうらおさんのyaneSDK4Dにあるものを利用させていただきました。
	http://www.sun-inet.or.jp/~yaneurao/dlang/

	個人的に利用しやすいようstatic化しただけで、中身は一緒です。
	@class CharCnv charcnv.d
**/
class CharCnv
{
	/// UTF8をSJISに変換する。
	/**
		@param[in]	UTF-8の文字列
	**/
	static char[] utf8ToSjis(char[] aUtf8)
	{
		return utf16ToSjis(toUTF16(aUtf8));
	}
	
	/// SJISをUTF8に変換する。
	/**
		@param[in]	Shift_JISの文字列
	**/
	static char[] sjisToUtf8(char[] aSjis)
	{
		return toUTF8(sjisToUtf16(aSjis));
	}
	
	/// UTF16をSJISに変換する。
	/**
		@param[in]	UTF-16の文字列
	**/
	static char[] utf16ToSjis(wchar[] aUtf16)
	{
		char[] result;
		version(Win32)
		{
			result.length = 
				WideCharToMultiByte(
					0,
					0,
					cast(char*)aUtf16,
					aUtf16.length,
					null,
					0,
					null,
					null
					);
			WideCharToMultiByte(
				0,
				0,
				cast(char*)aUtf16,
				aUtf16.length,
				result,
				result.length,
				null,
				null
				);
		}
		else
		{
			synchronized (locale_sync_object)
			{
				char* o = setlocale(LC_ALL, null);
				setlocale(LC_ALL, "");
				result.length = wcstombs(null, s, 0);
				wcstombs(result, s, result.length);
				setlocale(LC_ALL, o);
			}
		}
		return result;
	}
	
	/// SJISをUTF16に変換する。
	/**
		@param[in]	Shift_JISの文字列
	**/
	static wchar[] sjisToUtf16(char[] aSjis)
	{
		wchar[] result;
		version(Win32)
		{
			result.length = 
				MultiByteToWideChar(
					0,
					0,
					cast(char*)aSjis,
					aSjis.length,
					null,
					0
					);
			MultiByteToWideChar(
				0,
				0,
				aSjis,
				aSjis.length,
				result,
				result.length
				);
		}
		else
		{
			synchronized (locale_sync_object)
			{
				char* o = setlocale(LC_ALL, null);
				setlocale(LC_ALL, "");
				result.length = mbstowcs(null, s, 0);
				mbstowcs(result, s, result.length);
				setlocale(LC_ALL, o);
			}
		}
		return result;
	}
	
	unittest
	{
		char[] sjis = "\x82\xA8\x82\xc1\x82\xaf\x81\x5b";
		char[] utf8 = "おっけー";
		char[] str = CharCnv.sjisToUtf8(sjis);
		assert(str == utf8);
		str = CharCnv.utf8ToSjis(utf8);
		assert(str == sjis);
	}
}

version(Win32)
{
	extern(Windows) int WideCharToMultiByte(
		 uint	  CodePage,
		 uint	 dwFlags,
		 char*	lpWideCharStr,
		 int	  cchWideChar,
		 char*	 lpMultiByteStr,
		 int	  cbMultiByte,
		 char*	 lpDefaultChar,
		 bool*	lpUsedDefaultChar);

	extern(Windows) int MultiByteToWideChar(
		 uint	  CodePage,
		 uint	 dwFlags,
		 char*	 lpMultiByteStr,
		 int	 cchMultiByte,
		 wchar*	 lpWideCharStr,
		 int		cchWideChar);
}
else
{
	extern(C){
		enum { LC_ALL = 0 }
		char* setlocale(int, char*);
		int mbstowcs(wchar*, char*, int);
		int wcstombs(char*, wchar*, int);
	}
	/// set localeでスレッド排他するための同期オブジェクト
	private static Object locale_sync_object;

}
